@@ -1,6 +1,7 @@
 /* Openswan config file parser (parser.h)
  * Copyright (C) 2001-2002 Mathieu Lafon - Arkoon Network Security
- * Copyright (C) 2003-2006 Michael Richardson <mcr@xelerance.com>
+ * Copyright (C) 2003-2007 Michael Richardson <mcr@xelerance.com>
+ * Copyright (C) 2007-2008 Paul Wouters <paul@xelerance.com>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -12,7 +13,6 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: keywords.h,v 1.16 2004/12/02 16:26:02 ken Exp $
  */
 
 #ifndef _KEYWORDS_H_
@@ -65,14 +65,13 @@ enum keyword_numeric_config_field {
     KBF_HIDETOS,
     KBF_UNIQUEIDS,
     KBF_PLUTOWAIT,
-    KBF_FORWARDCONTROL,
     KBF_OVERRIDEMTU,
     KBF_STRICTCRLPOLICY,
     KBF_NOCRSEND,
     KBF_NATTRAVERSAL,
+    KBF_DISABLEPORTFLOATING,
     KBF_KEEPALIVE,
     KBF_PLUTORESTARTONCRASH,
-    KBF_RPFILTER,
     KBF_CRLCHECKINTERVAL,
     KBF_KLIPSDEBUG,
     KBF_PLUTODEBUG,
@@ -100,6 +99,7 @@ enum keyword_numeric_config_field {
     KBF_CONNADDRFAMILY,
     KBF_FORCEBUSY, /* 40 */
     KBF_OVERLAPIP,
+    KBF_WARNIGNORE, /* to ignore obsoleted keywords */
     KBF_MAX         
 };
 
@@ -218,6 +218,7 @@ enum keyword_type {
     kt_idtype,             /* an ID type */
     kt_bitstring,          /* an encryption/authentication key */
     kt_comment,            /* a value which is a cooked comment */
+    kt_obsolete,           /* option that is obsoleted, allow keyword but warn and ignore */
 };
 
 #define NOT_ENUM NULL
