@@ -201,10 +201,10 @@ failed:
 int
 ocf_aes_cbc_encrypt(
 	aes_context *ctx,
-	__u8 (*input),
+	const __u8 (*input),
 	__u8 (*output),
 	long length,
-	__u8 (*ivec),
+	const __u8 (*ivec),
 	int enc)
 {
 	if (crypto_fd != -1) {
@@ -219,7 +219,7 @@ ocf_aes_cbc_encrypt(
 			output,
 			length,
 			ivec,
-			16);
+			(u_int32_t)16);
 		return length;
 	}
 	return length;
