@@ -92,6 +92,7 @@ struct keyword_enum_value kw_dpdaction_values[]={
     { "hold",    DPD_ACTION_HOLD},
     { "clear",   DPD_ACTION_CLEAR},
     { "restart",   DPD_ACTION_RESTART},
+    { "restart_by_peer",   DPD_ACTION_RESTART_BY_PEER},
 };
 
 struct keyword_enum_values kw_dpdaction_list=
@@ -339,7 +340,7 @@ struct keyword_def ipsec_conf_keywords_v2[]={
     {"overlapip",      kv_conn|kv_auto, kt_bool,   KBF_OVERLAPIP, NOT_ENUM},
     {"rekey",          kv_conn|kv_auto, kt_bool,   KBF_REKEY, NOT_ENUM},
     {"rekeymargin",    kv_conn|kv_auto, kt_time,   KBF_REKEYMARGIN,NOT_ENUM},
-    {"rekeyfuzz",      kv_conn|kv_auto, kt_time,   KBF_REKEYFUZZ,NOT_ENUM},
+    {"rekeyfuzz",      kv_conn|kv_auto, kt_percent,   KBF_REKEYFUZZ,NOT_ENUM},
     {"keyingtries",    kv_conn|kv_auto, kt_number, KBF_KEYINGTRIES,NOT_ENUM},
     {"ikelifetime",    kv_conn|kv_auto, kt_time,   KBF_IKELIFETIME,NOT_ENUM},
     {"disablearrivalcheck", kv_conn|kv_auto, kt_invertbool, KBF_ARRIVALCHECK,NOT_ENUM},
@@ -435,7 +436,7 @@ int parser_find_keyword(const char *s, YYSTYPE *lval)
 
     lval->s = NULL;
     /* if we found nothing */
-    if(k->keyname == NULL && (s[0]=='x' || s[0]=='X') && s[1]=='-')
+    if(k->keyname == NULL && (s[0]=='x' || s[0]=='X') && (s[1]=='-' || s[1] =='_'))
     {
 	k = &ipsec_conf_keyword_comment;
 	lval->k.string = strdup(s);
