@@ -221,7 +221,12 @@ load_authcerts(const char *type, const char *path, u_char auth_flags)
 	    {
 		cert_t cert;
 
-		if (load_cert(CERT_NONE, filelist[n]->d_name, TRUE,
+		if (load_cert(CERT_NONE, filelist[n]->d_name,
+#ifdef SINGLE_CONF_DIR
+			FALSE, /* too verbose in single conf dir */
+#else
+			TRUE,
+#endif
 			      type, &cert))
 		    add_authcert(cert.u.x509, auth_flags);
 
