@@ -261,142 +261,6 @@ extern int sysctl_ipsec_regress_pfkey_lossage;
 #endif /* _IPSEC_PARAM_H_ */
 
 /*
- * $Log: ipsec_param.h,v $
-<<<<<<< HEAD:linux/include/openswan/ipsec_param.h
- * Revision 1.31  2005/08/12 15:01:38  mcr
- * 	attempt to #undef CONFIG_IPSEC_NAT_TRAVERSAL if it is =0.
-=======
- * Revision 1.29.6.4  2007/09/05 02:30:06  paul
- * KLIPS_SATOT macro. Patch by David McCullough
- *
- * Revision 1.29.6.3  2006/05/01 14:32:31  mcr
- * added KLIPS_ERROR and make sure that things work without CONFIG_KLIPS_REGRESS.
- *
- * Revision 1.29.6.2  2005/11/27 21:40:14  paul
- * Pull down TTL fixes from head. this fixes "Unknown symbol sysctl_ip_default_ttl"
- * in for klips as module.
->>>>>>> 84dee3e...   patches for 2.6.22, skb-pointer-changes:linux/include/openswan/ipsec_param.h
- *
- * Revision 1.30  2005/08/05 08:50:45  mcr
- * 	move #include of skbuff.h to a place where
- * 	we know it will be kernel only code.
- *
- * Revision 1.29  2005/01/26 00:50:35  mcr
- * 	adjustment of confusion of CONFIG_IPSEC_NAT vs CONFIG_KLIPS_NAT,
- * 	and make sure that NAT_TRAVERSAL is set as well to match
- * 	userspace compiles of code.
- *
- * Revision 1.28  2004/09/13 15:50:15  mcr
- * 	spell NEED_INET properly, not NET_INET.
- *
- * Revision 1.27  2004/09/13 02:21:45  mcr
- * 	always turn on IPIP mode.
- * 	#define inet_protocol if necessary.
- *
- * Revision 1.26  2004/08/17 03:25:43  mcr
- * 	freeswan->openswan.
- *
- * Revision 1.25  2004/07/10 19:08:41  mcr
- * 	CONFIG_IPSEC -> CONFIG_KLIPS.
- *
- * Revision 1.24  2004/04/05 19:55:06  mcr
- * Moved from linux/include/freeswan/ipsec_param.h,v
- *
- * Revision 1.23  2003/12/13 19:10:16  mcr
- * 	refactored rcv and xmit code - same as FS 2.05.
- *
- * Revision 1.22  2003/10/31 02:27:05  mcr
- * 	pulled up port-selector patches and sa_id elimination.
- *
- * Revision 1.21.4.1  2003/10/29 01:10:19  mcr
- * 	elimited "struct sa_id"
- *
- * Revision 1.21  2003/04/03 17:38:18  rgb
- * Centralised ipsec_kfree_skb and ipsec_dev_{get,put}.
- * Change indentation for readability.
- *
- * Revision 1.20  2003/03/14 08:09:26  rgb
- * Fixed up CONFIG_IPSEC_DYNDEV definitions.
- *
- * Revision 1.19  2003/01/30 02:31:43  rgb
- *
- * Rename SAref table macro names for clarity.
- *
- * Revision 1.18  2002/09/30 19:06:26  rgb
- * 	Reduce default table to 16 bits width.
- *
- * Revision 1.17  2002/09/20 15:40:29  rgb
- * Define switch to activate new SAref code.
- * Prefix macros with "IPSEC_".
- * Rework saref freelist.
- * Restrict some bits to kernel context for use to klips utils.
- *
- * Revision 1.16  2002/09/20 05:00:31  rgb
- * Define switch to divulge hmac keys for debugging.
- * Added IPOPTIONS switch.
- *
- * Revision 1.15  2002/09/19 02:34:24  mcr
- * 	define IPSEC_PROC_SUBDIRS if we are 2.4, and use that in ipsec_proc.c
- * 	to decide if we are to create /proc/net/ipsec/.
- *
- * Revision 1.14  2002/08/30 01:20:54  mcr
- * 	reorganized 2.0/2.2/2.4 procfs support macro so match
- * 	2.4 values/typedefs.
- *
- * Revision 1.13  2002/07/28 22:03:28  mcr
- * 	added some documentation to SA_REF_*
- * 	turned on fix for ESPIV attack, now that we have the attack code.
- *
- * Revision 1.12  2002/07/26 08:48:31  rgb
- * Added SA ref table code.
- *
- * Revision 1.11  2002/07/23 02:57:45  rgb
- * Define ARPHRD_VOID for < 2.4 kernels.
- *
- * Revision 1.10  2002/05/27 21:37:28  rgb
- * Set the defaults sanely for those adventurous enough to try more than 1
- * digit of ipsec devices.
- *
- * Revision 1.9  2002/05/27 18:56:07  rgb
- * Convert to dynamic ipsec device allocation.
- *
- * Revision 1.8  2002/04/24 07:36:47  mcr
- * Moved from ./klips/net/ipsec/ipsec_param.h,v
- *
- * Revision 1.7  2002/04/20 00:12:25  rgb
- * Added esp IV CBC attack fix, disabled.
- *
- * Revision 1.6  2002/01/29 02:11:42  mcr
- * 	removal of kversions.h - sources that needed it now use ipsec_param.h.
- * 	updating of IPv6 structures to match latest in6.h version.
- * 	removed dead code from freeswan.h that also duplicated kversions.h
- * 	code.
- *
- * Revision 1.5  2002/01/28 19:22:01  mcr
- * 	by default, turn off LINEARIZE option
- * 	(let kversions.h turn it on)
- *
- * Revision 1.4  2002/01/20 20:19:36  mcr
- * 	renamed option to IP_FRAGMENT_LINEARIZE.
- *
- * Revision 1.3  2002/01/12 02:57:25  mcr
- * 	first regression test causes acquire messages to be lost
- * 	100% of the time. This is to help testing of pluto.
- *
- * Revision 1.2  2001/11/26 09:16:14  rgb
- * Merge MCR's ipsec_sa, eroute, proc and struct lifetime changes.
- *
- * Revision 1.1.2.3  2001/10/23 04:40:16  mcr
- * 	added #define for DIVULGING session keys in debug output.
- *
- * Revision 1.1.2.2  2001/10/22 20:53:25  mcr
- * 	added a define to control forcing of DES parity.
- *
- * Revision 1.1.2.1  2001/09/25 02:20:19  mcr
- * 	many common kernel configuration questions centralized.
- * 	more things remain that should be moved from freeswan.h.
- *
- *
  * Local variables:
  * c-file-style: "linux"
  * End:
