@@ -124,13 +124,17 @@ DEBUG_NO_STATIC int pfkey_recvmsg(struct socket *sock, struct msghdr *msg, int s
 #endif
 
 struct net_proto_family pfkey_family_ops = {
+#ifdef NET_26
         .owner  = THIS_MODULE, 
+#endif
         .family = PF_KEY,
         .create = pfkey_create
 };
 
 struct proto_ops SOCKOPS_WRAPPED(pfkey_ops) = {
+#ifdef NET_26
         owner:          THIS_MODULE,
+#endif
 	family:		PF_KEY,
 	release:	pfkey_release,
 	bind:		sock_no_bind,
@@ -1108,6 +1112,10 @@ pfkey_get_info(char *buffer, char **start, off_t offset, int length
 #ifndef  PROC_NO_DUMMY
 , int dummy
 #endif /* !PROC_NO_DUMMY */
+#ifdef  PROC_EOF_DATA
+, int *eof
+, void *data
+#endif
 )
 {
 	const int max_content = length > 0? length-1 : 0;	/* limit of useful snprintf output */
@@ -1203,6 +1211,10 @@ pfkey_supported_get_info(char *buffer, char **start, off_t offset, int length
 #ifndef  PROC_NO_DUMMY
 , int dummy
 #endif /* !PROC_NO_DUMMY */
+#ifdef  PROC_EOF_DATA
+, int *eof
+, void *data
+#endif
 )
 {
 	/* limit of useful snprintf output */
@@ -1219,7 +1231,7 @@ pfkey_supported_get_info(char *buffer, char **start, off_t offset, int length
 		ps = pfkey_supported_list[satype];
 		while(ps) {
 			struct ipsec_alg_supported *alg = ps->supportedp;
-			unsigned char *n = alg->ias_name;
+			const char *n = alg->ias_name;
 			if(n == NULL) n = "unknown";
 
 			len += ipsec_snprintf(buffer+len, length-len,
@@ -1263,6 +1275,10 @@ pfkey_registered_get_info(char *buffer, char **start, off_t offset, int length
 #ifndef  PROC_NO_DUMMY
 , int dummy
 #endif /* !PROC_NO_DUMMY */
+#ifdef  PROC_EOF_DATA
+, int *eof
+, void *data
+#endif
 )
 {
 	const int max_content = length > 0? length-1 : 0;	/* limit of useful snprintf output */
@@ -1351,7 +1367,7 @@ supported_add_all(int satype, struct ipsec_alg_supported supported[], int size)
 
 	for(i = 0; i < size / sizeof(struct ipsec_alg_supported); i++) {
 
-		unsigned char *n = supported[i].ias_name;
+		const char *n = supported[i].ias_name;
 		if(n == NULL) n="unknown";
 
 		KLIPS_PRINT(debug_pfkey,
@@ -1379,7 +1395,7 @@ supported_remove_all(int satype)
 	struct ipsec_alg_supported*supportedp;
 
 	while(pfkey_supported_list[satype]) {
-		unsigned char *n;
+		const char *n;
 		supportedp = pfkey_supported_list[satype]->supportedp;
 
 		n = supportedp->ias_name;
@@ -1406,6 +1422,10 @@ pfkey_init(void)
 {
 	int error = 0;
 	int i;
+#ifdef HAVE_PROC_DIR_ENTRY
+	struct proc_dir_entry* entry;
+#endif
+
 	
 	static struct ipsec_alg_supported supported_init_ah[] = {
 #ifdef CONFIG_KLIPS_AUTH_HMAC_MD5
@@ -1477,9 +1497,12 @@ pfkey_init(void)
 	proc_net_create ("pf_key_supported", 0, pfkey_supported_get_info);
 	proc_net_create ("pf_key_registered", 0, pfkey_registered_get_info);
 #    else
-	create_proc_entry ("pf_key", 0, init_net.proc_net);
-	create_proc_entry ("pf_key_supported", 0, init_net.proc_net);
-	create_proc_entry ("pf_key_registered", 0, init_net.proc_net);
+	entry = create_proc_entry ("pf_key", 0, init_net.proc_net);
+	entry->read_proc = pfkey_get_info;
+	entry = create_proc_entry ("pf_key_supported", 0, init_net.proc_net);
+	entry->read_proc = pfkey_supported_get_info;
+	entry = create_proc_entry ("pf_key_registered", 0, init_net.proc_net);
+	entry->read_proc = pfkey_registered_get_info;
 #    endif
 #  endif /* !PROC_FS_2325 */
 #endif /* CONFIG_PROC_FS */
