@@ -28,7 +28,6 @@
 
 #ifdef CONFIG_SYSCTL
 
-#define NET_IPSEC 2112 /* Random number */                                        
 #ifdef CONFIG_KLIPS_DEBUG
 extern int       debug_ah;
 extern int       debug_esp;
@@ -53,7 +52,33 @@ extern int sysctl_ipsec_inbound_policy_check;
 extern int sysctl_ipsec_tos;
 int sysctl_ipsec_regress_pfkey_lossage;
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,24)
+#define NET_IPSEC CTL_UNNUMBERED
+#ifdef CONFIG_KLIPS_DEBUG
+	#define NET_IPSEC_DEBUG_AH		CTL_UNNUMBERED
+	#define NET_IPSEC_DEBUG_ESP		CTL_UNNUMBERED
+	#define NET_IPSEC_DEBUG_TUNNEL		CTL_UNNUMBERED
+	#define NET_IPSEC_DEBUG_EROUTE		CTL_UNNUMBERED
+	#define NET_IPSEC_DEBUG_SPI		CTL_UNNUMBERED
+	#define NET_IPSEC_DEBUG_RADIJ		CTL_UNNUMBERED
+	#define NET_IPSEC_DEBUG_NETLINK		CTL_UNNUMBERED
+	#define NET_IPSEC_DEBUG_XFORM		CTL_UNNUMBERED
+	#define NET_IPSEC_DEBUG_RCV		CTL_UNNUMBERED
+	#define NET_IPSEC_DEBUG_PFKEY		CTL_UNNUMBERED
+	#define NET_IPSEC_DEBUG_VERBOSE		CTL_UNNUMBERED
+	#define NET_IPSEC_DEBUG_IPCOMP		CTL_UNNUMBERED
+#endif /* CONFIG_KLIPS_DEBUG */
+	#define NET_IPSEC_ICMP			CTL_UNNUMBERED
+	#define NET_IPSEC_INBOUND_POLICY_CHECK	CTL_UNNUMBERED
+	#define NET_IPSEC_TOS			CTL_UNNUMBERED
+	#define NET_IPSEC_REGRESS_PFKEY_LOSSAGE	CTL_UNNUMBERED
+	#define NET_IPSEC_DEBUG_MAST		CTL_UNNUMBERED
+	#define NET_IPSEC_DEBUG_XMIT		CTL_UNNUMBERED
+#else
 enum {
+
+#define NET_IPSEC 2112 /* Random number - if not blame linux sysctl people */ 
+
 #ifdef CONFIG_KLIPS_DEBUG
 	NET_IPSEC_DEBUG_AH=1,
 	NET_IPSEC_DEBUG_ESP=2,
@@ -75,8 +100,10 @@ enum {
 	NET_IPSEC_DEBUG_MAST=17,
 	NET_IPSEC_DEBUG_XMIT=18,
 };
+#endif
 
 static ctl_table ipsec_table[] = {
+
 #ifdef CONFIG_KLIPS_DEBUG
 #ifdef CTL_TABLE_PARENT
        { .ctl_name = NET_IPSEC_DEBUG_AH,
