@@ -250,6 +250,9 @@ struct connection {
 #ifdef XAUTH_USEPAM 
     pam_handle_t  *pamh;		/*  PAM handle for that connection  */
 #endif
+#ifdef DYNAMICDNS
+    char *dnshostname;
+#endif /* DYNAMICDNS */
 #ifdef MODECFG
     ip_address modecfg_dns1;
     ip_address modecfg_dns2;
@@ -279,6 +282,7 @@ extern void initiate_connection(const char *name
 				, int whackfd
 				, lset_t moredebug
 				, enum crypto_importance importance);
+extern void restart_connections_by_peer(struct connection *c);
 extern void initiate_ondemand(const ip_address *our_client
 			      , const ip_address *peer_client
 			      , int transport_proto
@@ -398,14 +402,14 @@ extern void show_connections_status(void);
 extern int  connection_compare(const struct connection *ca
 			       , const struct connection *cb);
 #ifdef NAT_TRAVERSAL
-void
+extern void
 update_host_pair(const char *why, struct connection *c,
        const ip_address *myaddr, u_int16_t myport ,
        const ip_address *hisaddr, u_int16_t hisport);
 #endif /* NAT_TRAVERSAL */
 
 /* export to pending.c */
-void host_pair_enqueue_pending(const struct connection *c
+extern void host_pair_enqueue_pending(const struct connection *c
 			       , struct pending *p
 			       , struct pending **pnext);
 struct pending **host_pair_first_pending(const struct connection *c);
@@ -426,7 +430,9 @@ extern int foreach_connection_by_alias(const char *alias
 				       , void *arg);
 
 
-struct connection *unoriented_connections;
+extern struct connection *unoriented_connections;
+
+extern void update_host_pairs(struct connection *c);
 
 /*
  * Local Variables:
