@@ -30,6 +30,7 @@
 #ifndef _PLUTO_CRYPT_H
 #define _PLUTO_CRYPT_H
 
+#include "osw_select.h"
 #include "crypto.h"
 
 typedef unsigned int pcr_req_id;
@@ -156,8 +157,8 @@ extern void init_crypto_helpers(int nhelpers);
 extern err_t send_crypto_helper_request(struct pluto_crypto_req *r
 					, struct pluto_crypto_req_cont *cn
 					, bool *toomuch);
-extern void pluto_crypto_helper_sockets(fd_set *readfds);
-extern int  pluto_crypto_helper_ready(fd_set *readfds);
+extern void pluto_crypto_helper_sockets(osw_fd_set *readfds);
+extern int  pluto_crypto_helper_ready(osw_fd_set *readfds);
 
 extern void pluto_do_crypto_op(struct pluto_crypto_req *r);
 extern void pluto_crypto_helper(int fd, int helpernum);
