@@ -87,6 +87,8 @@
 #include "nat_traversal.h"
 #endif
 
+#include "osw_select.h"
+
 /*
  *  Server main loop and socket initialization routines.
  */
@@ -563,8 +565,8 @@ call_server(void)
 
     for (;;)
     {
-	fd_set readfds;
-	fd_set writefds;
+	osw_fd_set readfds;
+	osw_fd_set writefds;
 	int ndes;
 
 	/* wait for next interesting thing */
@@ -592,11 +594,11 @@ call_server(void)
 		reapchildren();
 	    }
 
-	    FD_ZERO(&readfds);
-	    FD_ZERO(&writefds);
-	    FD_SET(ctl_fd, &readfds);
+	    OSW_FD_ZERO(&readfds);
+	    OSW_FD_ZERO(&writefds);
+	    OSW_FD_SET(ctl_fd, &readfds);
 #ifdef IPSECPOLICY
-	    FD_SET(info_fd, &readfds);
+	    OSW_FD_SET(info_fd, &readfds);
 	    if (maxfd < info_fd)
 		maxfd = info_fd;
 #endif
@@ -606,14 +608,14 @@ call_server(void)
 	    {
 		if (maxfd < adns_qfd)
 		    maxfd = adns_qfd;
-		FD_SET(adns_qfd, &writefds);
+		OSW_FD_SET(adns_qfd, &writefds);
 	    }
 
 	    if (adns_afd != NULL_FD)
 	    {
 		if (maxfd < adns_afd)
 		    maxfd = adns_afd;
-		FD_SET(adns_afd, &readfds);
+		OSW_FD_SET(adns_afd, &readfds);
 	    }
 
 #ifdef KLIPS
@@ -625,8 +627,8 @@ call_server(void)
 		    kernel_ops->process_queue();
 		if (maxfd < fd)
 		    maxfd = fd;
-		passert(!FD_ISSET(fd, &readfds));
-		FD_SET(fd, &readfds);
+		passert(!OSW_FD_ISSET(fd, &readfds));
+		OSW_FD_SET(fd, &readfds);
 	    }
 #endif
 
@@ -636,8 +638,8 @@ call_server(void)
 		{
 		    if (maxfd < ifp->fd)
 			maxfd = ifp->fd;
-		    passert(!FD_ISSET(ifp->fd, &readfds));
-		    FD_SET(ifp->fd, &readfds);
+		    passert(!OSW_FD_ISSET(ifp->fd, &readfds));
+		    OSW_FD_SET(ifp->fd, &readfds);
 		}
 	    }
 
@@ -648,7 +650,7 @@ call_server(void)
 	    {
 		/* select without timer */
 
-		ndes = select(maxfd + 1, &readfds, &writefds, NULL, NULL);
+		ndes = osw_select(maxfd + 1, &readfds, &writefds, NULL, NULL);
 	    }
 	    else if (next_time == 0)
 	    {
@@ -665,7 +667,7 @@ call_server(void)
 
 		tm.tv_sec = next_time;
 		tm.tv_usec = 0;
-		ndes = select(maxfd + 1, &readfds, &writefds, NULL, &tm);
+		ndes = osw_select(maxfd + 1, &readfds, &writefds, NULL, &tm);
 	    }
 
 	    if (ndes != -1)
@@ -718,7 +720,7 @@ call_server(void)
 	{
 	    /* at least one file descriptor is ready */
 
-	    if (adns_qfd != NULL_FD && FD_ISSET(adns_qfd, &writefds))
+	    if (adns_qfd != NULL_FD && OSW_FD_ISSET(adns_qfd, &writefds))
 	    {
 		passert(ndes > 0);
 		send_unsent_ADNS_queries();
@@ -726,7 +728,7 @@ call_server(void)
 		ndes--;
 	    }
 
-	    if (adns_afd != NULL_FD && FD_ISSET(adns_afd, &readfds))
+	    if (adns_afd != NULL_FD && OSW_FD_ISSET(adns_afd, &readfds))
 	    {
 		passert(ndes > 0);
 		DBG(DBG_CONTROL,
@@ -738,7 +740,7 @@ call_server(void)
 
 #ifdef KLIPS
 	    if (kern_interface != NO_KERNEL
-		&& FD_ISSET(*kernel_ops->async_fdp, &readfds))
+		&& OSW_FD_ISSET(*kernel_ops->async_fdp, &readfds))
 	    {
 		passert(ndes > 0);
 		DBG(DBG_CONTROL,
@@ -751,7 +753,7 @@ call_server(void)
 
 	    for (ifp = interfaces; ifp != NULL; ifp = ifp->next)
 	    {
-		if (FD_ISSET(ifp->fd, &readfds))
+		if (OSW_FD_ISSET(ifp->fd, &readfds))
 		{
 		    /* comm_handle will print DBG_CONTROL intro,
 		     * with more info than we have here.
@@ -764,7 +766,7 @@ call_server(void)
 		}
 	    }
 
-	    if (FD_ISSET(ctl_fd, &readfds))
+	    if (OSW_FD_ISSET(ctl_fd, &readfds))
 	    {
 		passert(ndes > 0);
 		DBG(DBG_CONTROL,
@@ -775,7 +777,7 @@ call_server(void)
 	    }
 
 #ifdef IPSECPOLICY
-	    if (FD_ISSET(info_fd, &readfds))
+	    if (OSW_FD_ISSET(info_fd, &readfds))
 	    {
 		passert(ndes > 0);
 		DBG(DBG_CONTROL,
