@@ -119,6 +119,7 @@ static struct db_attr otpsk1024aesmd5[] = {
 	{ .type.oakley=OAKLEY_HASH_ALGORITHM, .val=OAKLEY_MD5 },
 	{ .type.oakley=OAKLEY_AUTHENTICATION_METHOD, .val=OAKLEY_PRESHARED_KEY },
 	{ .type.oakley=OAKLEY_GROUP_DESCRIPTION, .val=OAKLEY_GROUP_MODP1024 },
+	{ .type.oakley=OAKLEY_KEY_LENGTH, .val=128 },
 	}; 
 
 static struct db_attr otpsk1536aesmd5[] = {
@@ -126,6 +127,7 @@ static struct db_attr otpsk1536aesmd5[] = {
 	{ .type.oakley=OAKLEY_HASH_ALGORITHM, .val=OAKLEY_MD5 },
 	{ .type.oakley=OAKLEY_AUTHENTICATION_METHOD, .val=OAKLEY_PRESHARED_KEY },
 	{ .type.oakley=OAKLEY_GROUP_DESCRIPTION, .val=OAKLEY_GROUP_MODP1536 },
+	{ .type.oakley=OAKLEY_KEY_LENGTH, .val=128 },
 	}; 
 
 static struct db_attr otpsk2048aesmd5[] = {
@@ -133,6 +135,7 @@ static struct db_attr otpsk2048aesmd5[] = {
 	{ .type.oakley=OAKLEY_HASH_ALGORITHM, .val=OAKLEY_MD5 },
 	{ .type.oakley=OAKLEY_AUTHENTICATION_METHOD, .val=OAKLEY_PRESHARED_KEY },
 	{ .type.oakley=OAKLEY_GROUP_DESCRIPTION, .val=OAKLEY_GROUP_MODP2048 },
+	{ .type.oakley=OAKLEY_KEY_LENGTH, .val=128 },
 	}; 
 
 static struct db_attr otpsk1024aessha1[] = {
@@ -140,6 +143,7 @@ static struct db_attr otpsk1024aessha1[] = {
 	{ .type.oakley=OAKLEY_HASH_ALGORITHM, .val=OAKLEY_SHA1 },
 	{ .type.oakley=OAKLEY_AUTHENTICATION_METHOD, .val=OAKLEY_PRESHARED_KEY },
 	{ .type.oakley=OAKLEY_GROUP_DESCRIPTION, .val=OAKLEY_GROUP_MODP1024 },
+	{ .type.oakley=OAKLEY_KEY_LENGTH, .val=128 },
 	};
 
 static struct db_attr otpsk1536aessha1[] = {
@@ -147,6 +151,7 @@ static struct db_attr otpsk1536aessha1[] = {
 	{ .type.oakley=OAKLEY_HASH_ALGORITHM, .val=OAKLEY_SHA1 },
 	{ .type.oakley=OAKLEY_AUTHENTICATION_METHOD, .val=OAKLEY_PRESHARED_KEY },
 	{ .type.oakley=OAKLEY_GROUP_DESCRIPTION, .val=OAKLEY_GROUP_MODP1536 },
+	{ .type.oakley=OAKLEY_KEY_LENGTH, .val=128 },
 	};
 
 static struct db_attr otpsk2048aessha1[] = {
@@ -154,6 +159,7 @@ static struct db_attr otpsk2048aessha1[] = {
 	{ .type.oakley=OAKLEY_HASH_ALGORITHM, .val=OAKLEY_SHA1 },
 	{ .type.oakley=OAKLEY_AUTHENTICATION_METHOD, .val=OAKLEY_PRESHARED_KEY },
 	{ .type.oakley=OAKLEY_GROUP_DESCRIPTION, .val=OAKLEY_GROUP_MODP2048 },
+	{ .type.oakley=OAKLEY_KEY_LENGTH, .val=128 },
 	};
 
 static struct db_attr otpsk1024des3sha1[] = {
@@ -212,6 +218,7 @@ static struct db_attr otpsk1536aesmd5_xauthc[] = {
 	{ .type.oakley=OAKLEY_HASH_ALGORITHM, .val=OAKLEY_MD5 },
 	{ .type.oakley=OAKLEY_AUTHENTICATION_METHOD, .val=XAUTHInitPreShared },
 	{ .type.oakley=OAKLEY_GROUP_DESCRIPTION, .val=OAKLEY_GROUP_MODP1536 },
+	{ .type.oakley=OAKLEY_KEY_LENGTH, .val=128 },
 	};
 
 static struct db_attr otpsk1536aessha1_xauthc[] = {
@@ -219,6 +226,7 @@ static struct db_attr otpsk1536aessha1_xauthc[] = {
 	{ .type.oakley=OAKLEY_HASH_ALGORITHM, .val=OAKLEY_SHA1 },
 	{ .type.oakley=OAKLEY_AUTHENTICATION_METHOD, .val=XAUTHInitPreShared },
 	{ .type.oakley=OAKLEY_GROUP_DESCRIPTION, .val=OAKLEY_GROUP_MODP1536 },
+	{ .type.oakley=OAKLEY_KEY_LENGTH, .val=128 },
 	};
 
 static struct db_attr otpsk1024des3md5_xauths[] = {
@@ -254,6 +262,7 @@ static struct db_attr otpsk1536aesmd5_xauths[] = {
 	{ .type.oakley=OAKLEY_HASH_ALGORITHM, .val=OAKLEY_MD5 },
 	{ .type.oakley=OAKLEY_AUTHENTICATION_METHOD, .val=XAUTHRespPreShared },
 	{ .type.oakley=OAKLEY_GROUP_DESCRIPTION, .val=OAKLEY_GROUP_MODP1536 },
+	{ .type.oakley=OAKLEY_KEY_LENGTH, .val=128 },
 	};
 
 static struct db_attr otpsk1536aessha1_xauths[] = {
@@ -261,6 +270,7 @@ static struct db_attr otpsk1536aessha1_xauths[] = {
 	{ .type.oakley=OAKLEY_HASH_ALGORITHM, .val=OAKLEY_SHA1 },
 	{ .type.oakley=OAKLEY_AUTHENTICATION_METHOD, .val=XAUTHRespPreShared },
 	{ .type.oakley=OAKLEY_GROUP_DESCRIPTION, .val=OAKLEY_GROUP_MODP1536 },
+	{ .type.oakley=OAKLEY_KEY_LENGTH, .val=128 },
 	};
 #endif
 
@@ -271,6 +281,7 @@ static struct db_attr otrsasig1024aesmd5[] = {
 	{ .type.oakley=OAKLEY_HASH_ALGORITHM, .val=OAKLEY_MD5 },
 	{ .type.oakley=OAKLEY_AUTHENTICATION_METHOD, .val=OAKLEY_RSA_SIG },
 	{ .type.oakley=OAKLEY_GROUP_DESCRIPTION, .val=OAKLEY_GROUP_MODP1024 },
+	{ .type.oakley=OAKLEY_KEY_LENGTH, .val=128 },
 	};
 
 static struct db_attr otrsasig1536aesmd5[] = {
@@ -278,6 +289,7 @@ static struct db_attr otrsasig1536aesmd5[] = {
 	{ .type.oakley=OAKLEY_HASH_ALGORITHM, .val=OAKLEY_MD5 },
 	{ .type.oakley=OAKLEY_AUTHENTICATION_METHOD, .val=OAKLEY_RSA_SIG },
 	{ .type.oakley=OAKLEY_GROUP_DESCRIPTION, .val=OAKLEY_GROUP_MODP1536 },
+	{ .type.oakley=OAKLEY_KEY_LENGTH, .val=128 },
 	};
 
 static struct db_attr otrsasig2048aesmd5[] = {
@@ -285,6 +297,7 @@ static struct db_attr otrsasig2048aesmd5[] = {
 	{ .type.oakley=OAKLEY_HASH_ALGORITHM, .val=OAKLEY_MD5 },
 	{ .type.oakley=OAKLEY_AUTHENTICATION_METHOD, .val=OAKLEY_RSA_SIG },
 	{ .type.oakley=OAKLEY_GROUP_DESCRIPTION, .val=OAKLEY_GROUP_MODP2048 },
+	{ .type.oakley=OAKLEY_KEY_LENGTH, .val=128 },
 	};
 
 static struct db_attr otrsasig1024aessha1[] = {
@@ -292,6 +305,7 @@ static struct db_attr otrsasig1024aessha1[] = {
 	{ .type.oakley=OAKLEY_HASH_ALGORITHM, .val=OAKLEY_SHA1 },
 	{ .type.oakley=OAKLEY_AUTHENTICATION_METHOD, .val=OAKLEY_RSA_SIG },
 	{ .type.oakley=OAKLEY_GROUP_DESCRIPTION, .val=OAKLEY_GROUP_MODP1024 },
+	{ .type.oakley=OAKLEY_KEY_LENGTH, .val=128 },
 	};
 
 static struct db_attr otrsasig1536aessha1[] = {
@@ -299,6 +313,7 @@ static struct db_attr otrsasig1536aessha1[] = {
 	{ .type.oakley=OAKLEY_HASH_ALGORITHM, .val=OAKLEY_SHA1 },
 	{ .type.oakley=OAKLEY_AUTHENTICATION_METHOD, .val=OAKLEY_RSA_SIG },
 	{ .type.oakley=OAKLEY_GROUP_DESCRIPTION, .val=OAKLEY_GROUP_MODP1536 },
+	{ .type.oakley=OAKLEY_KEY_LENGTH, .val=128 },
 	};
 
 static struct db_attr otrsasig2048aessha1[] = {
@@ -306,6 +321,7 @@ static struct db_attr otrsasig2048aessha1[] = {
 	{ .type.oakley=OAKLEY_HASH_ALGORITHM, .val=OAKLEY_SHA1 },
 	{ .type.oakley=OAKLEY_AUTHENTICATION_METHOD, .val=OAKLEY_RSA_SIG },
 	{ .type.oakley=OAKLEY_GROUP_DESCRIPTION, .val=OAKLEY_GROUP_MODP2048 },
+	{ .type.oakley=OAKLEY_KEY_LENGTH, .val=128 },
 	};
 
 static struct db_attr otrsasig1024des3md5[] = {
@@ -372,6 +388,7 @@ static struct db_attr otrsasig1536aesmd5_xauthc[] = {
 	{ .type.oakley=OAKLEY_HASH_ALGORITHM, .val=OAKLEY_MD5 },
 	{ .type.oakley=OAKLEY_AUTHENTICATION_METHOD, .val=XAUTHInitRSA },
 	{ .type.oakley=OAKLEY_GROUP_DESCRIPTION, .val=OAKLEY_GROUP_MODP1536 },
+	{ .type.oakley=OAKLEY_KEY_LENGTH, .val=128 },
 	};
 
 static struct db_attr otrsasig1536aessha1_xauthc[] = {
@@ -379,6 +396,7 @@ static struct db_attr otrsasig1536aessha1_xauthc[] = {
 	{ .type.oakley=OAKLEY_HASH_ALGORITHM, .val=OAKLEY_SHA1 },
 	{ .type.oakley=OAKLEY_AUTHENTICATION_METHOD, .val=XAUTHInitRSA },
 	{ .type.oakley=OAKLEY_GROUP_DESCRIPTION, .val=OAKLEY_GROUP_MODP1536 },
+	{ .type.oakley=OAKLEY_KEY_LENGTH, .val=128 },
 	};
 
 static struct db_attr otrsasig1024des3sha1_xauthc[] = {
@@ -420,6 +438,7 @@ static struct db_attr otrsasig1536aesmd5_xauths[] = {
 	{ .type.oakley=OAKLEY_HASH_ALGORITHM, .val=OAKLEY_MD5 },
 	{ .type.oakley=OAKLEY_AUTHENTICATION_METHOD, .val=XAUTHInitRSA },
 	{ .type.oakley=OAKLEY_GROUP_DESCRIPTION, .val=OAKLEY_GROUP_MODP1536 },
+	{ .type.oakley=OAKLEY_KEY_LENGTH, .val=128 },
 	};
 
 static struct db_attr otrsasig1024des3sha1_xauths[] = {
@@ -441,6 +460,7 @@ static struct db_attr otrsasig1536aessha1_xauths[] = {
 	{ .type.oakley=OAKLEY_HASH_ALGORITHM, .val=OAKLEY_SHA1 },
 	{ .type.oakley=OAKLEY_AUTHENTICATION_METHOD, .val=XAUTHRespRSA },
 	{ .type.oakley=OAKLEY_GROUP_DESCRIPTION, .val=OAKLEY_GROUP_MODP1536 },
+	{ .type.oakley=OAKLEY_KEY_LENGTH, .val=128 },
 	};
 #endif
 
@@ -537,10 +557,14 @@ static struct db_trans oakley_trans_pskrsasig[] = {
 #ifdef TEST_INDECENT_PROPOSAL
 	{ AD_TR(KEY_IKE,otpsk1024des3tiger) },
 #endif
-	{ AD_TR(KEY_IKE,otrsasig1536des3md5) },
-	{ AD_TR(KEY_IKE,otpsk1536des3md5) },
+	{ AD_TR(KEY_IKE,otrsasig2048des3sha1) },
+	{ AD_TR(KEY_IKE,otpsk2048des3sha1) },
+	{ AD_TR(KEY_IKE,otrsasig2048des3md5) },
+	{ AD_TR(KEY_IKE,otpsk2048des3md5) },
 	{ AD_TR(KEY_IKE,otrsasig1536des3sha1) },
 	{ AD_TR(KEY_IKE,otpsk1536des3sha1) },
+	{ AD_TR(KEY_IKE,otrsasig1536des3md5) },
+	{ AD_TR(KEY_IKE,otpsk1536des3md5) },
 	{ AD_TR(KEY_IKE,otrsasig1024des3sha1) },
 	{ AD_TR(KEY_IKE,otpsk1024des3sha1) },
 	{ AD_TR(KEY_IKE,otrsasig1024des3md5) },
@@ -782,6 +806,16 @@ struct db_sa oakley_am_sadb[] = {
 
 /* arrays of attributes for transforms */
 
+static struct db_attr espamd5_attr[] = {
+    { .type.ipsec=AUTH_ALGORITHM, AUTH_ALGORITHM_HMAC_MD5 },
+    { .type.ipsec=KEY_LENGTH, 128 },
+    };
+
+static struct db_attr espasha1_attr[] = {
+    { .type.ipsec=AUTH_ALGORITHM, AUTH_ALGORITHM_HMAC_SHA1 },
+    { .type.ipsec=KEY_LENGTH, 128 },
+    };
+
 static struct db_attr espmd5_attr[] = {
     { .type.ipsec=AUTH_ALGORITHM, AUTH_ALGORITHM_HMAC_MD5 },
     };
@@ -801,8 +835,8 @@ static struct db_attr ah_HMAC_SHA1_attr[] = {
 /* arrays of transforms, each in in preference order */
 
 static struct db_trans espa_trans[] = {
-    { AD_TR(ESP_AES, espsha1_attr) },
-    { AD_TR(ESP_AES, espmd5_attr) },
+    { AD_TR(ESP_AES, espasha1_attr) },
+    { AD_TR(ESP_AES, espamd5_attr) },
     { AD_TR(ESP_3DES,espsha1_attr) },
     { AD_TR(ESP_3DES,espmd5_attr) },
     };
