@@ -265,6 +265,8 @@ insert_state(struct state *st)
      */
     if (st->st_event == NULL)
 	event_schedule(EVENT_SO_DISCARD, 0, st);
+
+	refresh_state(st);
 }
 
 /*
@@ -350,7 +352,6 @@ delete_state(struct state *st)
 
     DBG(DBG_CONTROL, DBG_log("deleting state #%lu", st->st_serialno));
 
-    set_cur_state(st);
 
     /* If DPD is enabled on this state object, clear any pending events */
     if(st->st_dpd_event != NULL)
@@ -404,6 +405,8 @@ delete_state(struct state *st)
     cur_state = old_cur_state;	/* without st_connection, st isn't complete */
     connection_discard(c);
 
+    change_state(st, STATE_UNDEFINED);
+
     release_whack(st);
 
     /* from here on we are just freeing RAM */
@@ -508,7 +511,7 @@ foreach_states_by_connection_func(struct connection *c
 		st = st->st_hashchain_next;	/* before this is deleted */
 
 		/* on pass 2, ignore phase2 states */
- 		if(pass == 2 && IS_ISAKMP_SA_ESTABLISHED(this->st_state)) {
+ 		if(pass == 1 && IS_ISAKMP_SA_ESTABLISHED(this->st_state)) {
 		    continue;
 		}
 
@@ -736,8 +739,12 @@ rekey_p2states_by_connection(struct connection *c)
 }
 
 
-/* Walk through the state table, and delete each state whose phase 1 (IKE)
+/*
+ * Walk through the state table, and delete each state whose phase 1 (IKE)
  * peer is among those given.
+ * TODO: This function is only called for ipsec whack --crash peer, but
+ * it currently does not work for IKEv2, since IS_PHASE1() only works on IKEv1
+ * Filed as bug http://bugs.xelerance.com/view.php?id=971
  */
 void
 delete_states_by_peer(ip_address *peer)
